<?php 
/**
 * Customer_addresses Page Controller
 * @category  Controller
 */
class Customer_addressesController extends SecureController{
	function __construct(){
		parent::__construct();
		$this->tablename = "customer_addresses";
		$this->soft_delete = true;
		$this->delete_field_name = "address_status";
		$this->delete_field_value = "deleted";
	}
	/**
     * List page records
     * @param $fieldname (filter record by a field) 
     * @param $fieldvalue (filter field value)
     * @return BaseView
     */
	function index($fieldname = null , $fieldvalue = null){
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$fields = array("customer_addresses.address_id", 
			"customer_addresses.customer_id", 
			"customers.customer_full_name AS customers_customer_full_name", 
			"customer_addresses.country_id", 
			"countries.country_name_en AS countries_country_name_en", 
			"customer_addresses.address_name", 
			"customer_addresses.address_area", 
			"customer_addresses.address_type", 
			"customer_addresses.created_at");
		$pagination = $this->get_pagination(MAX_RECORD_COUNT); // get current pagination e.g array(page_number, page_limit)
		$db->join("customers", "customer_addresses.customer_id = customers.customer_id", "INNER");
		$db->join("countries", "customer_addresses.country_id = countries.country_id", "INNER");
		if(!empty($request->orderby)){
			$orderby = $request->orderby;
			$ordertype = (!empty($request->ordertype) ? $request->ordertype : ORDER_TYPE);
			$db->orderBy($orderby, $ordertype);
		}
		else{
			$db->orderBy("customer_addresses.address_id", ORDER_TYPE);
		}
		if($fieldname){
			$db->where($fieldname , $fieldvalue); //filter by a single field name
		}
		$tc = $db->withTotalCount();
		$records = $db->get($tablename, $pagination, $fields);
		$records_count = count($records);
		$total_records = intval($tc->totalCount);
		$page_limit = $pagination[1];
		$total_pages = ceil($total_records / $page_limit);
		$data = new stdClass;
		$data->records = $records;
		$data->record_count = $records_count;
		$data->total_records = $total_records;
		$data->total_page = $total_pages;
		//list of record id for audit trail
		$this->rec_id = array_column($records, "address_id");
		$this->write_to_log("list", "true");
		if($db->getLastError()){
			$this->set_page_error();
			$this->write_to_log("list", "false");
		}
		$page_title = $this->view->page_title = get_lang('customer_addresses');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		$this->render_view("customer_addresses/list.php", $data); //render the full page
	}
	/**
     * View record detail 
	 * @param $rec_id (select record by table primary key) 
     * @param $value value (select record by value of field name(rec_id))
     * @return BaseView
     */
	function view($rec_id = null, $value = null){
		$request = $this->request;
		$db = $this->GetModel();
		$rec_id = $this->rec_id = urldecode($rec_id);
		$tablename = $this->tablename;
		$fields = array("customer_addresses.address_id", 
			"customer_addresses.customer_id", 
			"customers.customer_full_name AS customers_customer_full_name", 
			"customer_addresses.country_id", 
			"countries.country_name_en AS countries_country_name_en", 
			"customer_addresses.address_name", 
			"customer_addresses.address_area", 
			"customer_addresses.address_block", 
			"customer_addresses.address_road", 
			"customer_addresses.address_building", 
			"customer_addresses.address_latitude", 
			"customer_addresses.address_longitude", 
			"customer_addresses.address_floor", 
			"customer_addresses.address_room_number", 
			"customer_addresses.address_note", 
			"customer_addresses.address_type", 
			"customer_addresses.created_at", 
			"customer_addresses.updated_at");
		if($value){
			$db->where($rec_id, urldecode($value)); //select record based on field name
		}
		else{
			$db->where("customer_addresses.address_id", $rec_id);; //select record based on primary key
		}
		$db->join("customers", "customer_addresses.customer_id = customers.customer_id", "INNER");
		$db->join("countries", "customer_addresses.country_id = countries.country_id", "INNER");  
		$record = $db->getOne($tablename, $fields );
		if($record){
			$this->write_to_log("view", "true");
			$page_title = $this->view->page_title = get_lang('view_customer_addresses');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		}
		else{
			if($db->getLastError()){
				$this->set_page_error();
			}
			else{
				$this->set_page_error(get_lang('no_record_found'));
			}
			$this->write_to_log("view", "false");
		}
		return $this->render_view("customer_addresses/view.php", $record);
	}
	/**
     * Update record field as deleted
	 * Support multi delete by separating record id by comma.
     * @return BaseView
     */
	function delete($rec_id = null){
		Csrf::cross_check();
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$this->rec_id = $rec_id;
		//form multiple delete, split record id separated by comma into array
		$arr_rec_id = array_map('trim', explode(",", $rec_id));
		$db->where("customer_addresses.address_id", $arr_rec_id, "in");
		$modeldata = array(
			"address_status" => "deleted",
			"deleted_at" => datetime_now()
		);
		$bool = $db->update($tablename, $modeldata);
		if($bool){
			$this->write_to_log("delete", "true");
			$this->set_flash_msg(get_lang('record_deleted_successfully'), "success");
		}
		else{
			$page_error = $db->getLastError();
			$this->write_to_log("delete", "false");
			$this->set_flash_msg($page_error, "danger");
		}
		return	$this->redirect("customer_addresses");
	}
}
