<?php 
/**
 * Products Page Controller
 * @category  Controller
 */
class ProductsController extends SecureController{
	function __construct(){
		parent::__construct();
		$this->tablename = "products";
		$this->soft_delete = true;
		$this->delete_field_name = "product_status";
		$this->delete_field_value = "deleted";
	}
	/**
     * List page records
     * @param $fieldname (filter record by a field) 
     * @param $fieldvalue (filter field value)
     * @return BaseView
     */
	function index($fieldname = null , $fieldvalue = null){
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$fields = array("products.product_id", 
			"products.product_name_en", 
			"products.product_name_ar", 
			"products.product_preview_img", 
			"products.product_price", 
			"products.category_id", 
			"sector_categories.category_name_en AS sector_categories_category_name_en", 
			"products.brand_id", 
			"brands.brand_name_en AS brands_brand_name_en", 
			"products.country_id", 
			"countries.country_name_en AS countries_country_name_en", 
			"products.created_at");
		$pagination = $this->get_pagination(MAX_RECORD_COUNT); // get current pagination e.g array(page_number, page_limit)
		//search table record
		if(!empty($request->search)){
			$text = trim($request->search); 
			$search_condition = "(
				products.product_name_en LIKE ? OR 
				products.product_name_ar LIKE ? OR 
				products.product_description_en LIKE ? OR 
				products.product_description_ar LIKE ? OR 
				products.product_price LIKE ? OR 
				products.deleted_at LIKE ?
			)";
			$search_params = array(
				"%$text%","%$text%","%$text%","%$text%","%$text%","%$text%"
			);
			//setting search conditions
			$db->where($search_condition, $search_params);
			 //template to use when ajax search
			$this->view->search_template = "products/search.php";
		}
		$db->join("sector_categories", "products.category_id = sector_categories.category_id", "INNER");
		$db->join("brands", "products.brand_id = brands.brand_id", "INNER");
		$db->join("countries", "products.country_id = countries.country_id", "INNER");
		if(!empty($request->orderby)){
			$orderby = $request->orderby;
			$ordertype = (!empty($request->ordertype) ? $request->ordertype : ORDER_TYPE);
			$db->orderBy($orderby, $ordertype);
		}
		else{
			$db->orderBy("products.product_id", ORDER_TYPE);
		}
		$db->where("products.created_by_admin_id", get_active_user('user_id') );
		if($fieldname){
			$db->where($fieldname , $fieldvalue); //filter by a single field name
		}
		$tc = $db->withTotalCount();
		$records = $db->get($tablename, $pagination, $fields);
		$records_count = count($records);
		$total_records = intval($tc->totalCount);
		$page_limit = $pagination[1];
		$total_pages = ceil($total_records / $page_limit);
		$data = new stdClass;
		$data->records = $records;
		$data->record_count = $records_count;
		$data->total_records = $total_records;
		$data->total_page = $total_pages;
		//list of record id for audit trail
		$this->rec_id = array_column($records, "product_id");
		$this->write_to_log("list", "true");
		if($db->getLastError()){
			$this->set_page_error();
			$this->write_to_log("list", "false");
		}
		$page_title = $this->view->page_title = get_lang('products');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		$this->render_view("products/list.php", $data); //render the full page
	}
	/**
     * View record detail 
	 * @param $rec_id (select record by table primary key) 
     * @param $value value (select record by value of field name(rec_id))
     * @return BaseView
     */
	function view($rec_id = null, $value = null){
		$request = $this->request;
		$db = $this->GetModel();
		$rec_id = $this->rec_id = urldecode($rec_id);
		$tablename = $this->tablename;
		$fields = array("products.product_id", 
			"products.product_name_en", 
			"products.product_name_ar", 
			"products.product_description_en", 
			"products.product_description_ar", 
			"products.product_preview_img", 
			"products.product_price", 
			"products.category_id", 
			"sector_categories.category_name_en AS sector_categories_category_name_en", 
			"products.brand_id", 
			"brands.brand_name_en AS brands_brand_name_en", 
			"products.country_id", 
			"countries.country_name_en AS countries_country_name_en", 
			"products.created_at", 
			"products.updated_at");
		$db->where("products.created_by_admin_id", get_active_user('user_id') );
		if($value){
			$db->where($rec_id, urldecode($value)); //select record based on field name
		}
		else{
			$db->where("products.product_id", $rec_id);; //select record based on primary key
		}
		$db->join("sector_categories", "products.category_id = sector_categories.category_id", "INNER");
		$db->join("brands", "products.brand_id = brands.brand_id", "INNER");
		$db->join("countries", "products.country_id = countries.country_id", "INNER");  
		$record = $db->getOne($tablename, $fields );
		if($record){
			$this->write_to_log("view", "true");
			$page_title = $this->view->page_title = get_lang('view_products');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		}
		else{
			if($db->getLastError()){
				$this->set_page_error();
			}
			else{
				$this->set_page_error(get_lang('no_record_found'));
			}
			$this->write_to_log("view", "false");
		}
		return $this->render_view("products/view.php", $record);
	}
	/**
     * Insert new record to the database table
	 * @param $formdata array() from $_POST
     * @return BaseView
     */
	function add($formdata = null){
		if($formdata){
			$db = $this->GetModel();
			$tablename = $this->tablename;
			$request = $this->request;
			//fillable fields
			$fields = $this->fields = array("product_id","category_id","brand_id","country_id","product_name_en","product_name_ar","product_description_en","product_description_ar","product_preview_img","product_price");
			$postdata = $this->format_request_data($formdata);
			$this->rules_array = array(
				'product_id' => 'required',
				'category_id' => 'required',
				'brand_id' => 'required',
				'country_id' => 'required',
				'product_name_en' => 'required',
				'product_name_ar' => 'required',
				'product_description_en' => 'required',
				'product_description_ar' => 'required',
				'product_preview_img' => 'required',
				'product_price' => 'required|numeric',
			);
			$this->sanitize_array = array(
				'product_id' => 'sanitize_string',
				'category_id' => 'sanitize_string',
				'brand_id' => 'sanitize_string',
				'country_id' => 'sanitize_string',
				'product_name_en' => 'sanitize_string',
				'product_name_ar' => 'sanitize_string',
				'product_description_en' => 'sanitize_string',
				'product_description_ar' => 'sanitize_string',
				'product_preview_img' => 'sanitize_string',
				'product_price' => 'sanitize_string',
			);
			$this->filter_vals = true; //set whether to remove empty fields
			$modeldata = $this->modeldata = $this->validate_form($postdata);
			$modeldata['created_at'] = datetime_now();
			//Check if Duplicate Record Already Exit In The Database
			$db->where("product_id", $modeldata['product_id']);
			if($db->has($tablename)){
				$this->view->page_error[] = $modeldata['product_id'].get_lang('_already_exist_');
			} 
			if($this->validated()){
				$rec_id = $this->rec_id = $db->insert($tablename, $modeldata);
				if($rec_id){
					$this->write_to_log("add", "true");
					$this->set_flash_msg(get_lang('record_added_successfully'), "success");
					return	$this->redirect("products");
				}
				else{
					$this->set_page_error();
					$this->write_to_log("add", "false");
				}
			}
		}
		$page_title = $this->view->page_title = get_lang('add_new_products');
		$this->render_view("products/add.php");
	}
	/**
     * Update table record with formdata
	 * @param $rec_id (select record by table primary key)
	 * @param $formdata array() from $_POST
     * @return array
     */
	function edit($rec_id = null, $formdata = null){
		$request = $this->request;
		$db = $this->GetModel();
		$this->rec_id = $rec_id;
		$tablename = $this->tablename;
		 //editable fields
		$fields = $this->fields = array("product_id","category_id","brand_id","country_id","product_name_en","product_name_ar","product_description_en","product_description_ar","product_preview_img","product_price");
		if($formdata){
			$postdata = $this->format_request_data($formdata);
			$this->rules_array = array(
				'category_id' => 'required',
				'brand_id' => 'required',
				'country_id' => 'required',
				'product_name_en' => 'required',
				'product_name_ar' => 'required',
				'product_description_en' => 'required',
				'product_description_ar' => 'required',
				'product_preview_img' => 'required',
				'product_price' => 'required|numeric',
			);
			$this->sanitize_array = array(
				'category_id' => 'sanitize_string',
				'brand_id' => 'sanitize_string',
				'country_id' => 'sanitize_string',
				'product_name_en' => 'sanitize_string',
				'product_name_ar' => 'sanitize_string',
				'product_description_en' => 'sanitize_string',
				'product_description_ar' => 'sanitize_string',
				'product_preview_img' => 'sanitize_string',
				'product_price' => 'sanitize_string',
			);
			$modeldata = $this->modeldata = $this->validate_form($postdata);
			$modeldata['updated_at'] = datetime_now();
			if($this->validated()){
		$db->where("products.created_by_admin_id", get_active_user('user_id') );
				$db->where("products.product_id", $rec_id);;
				$bool = $db->update($tablename, $modeldata);
				$numRows = $db->getRowCount(); //number of affected rows. 0 = no record field updated
				if($bool && $numRows){
					$this->write_to_log("edit", "true");
					$this->set_flash_msg(get_lang('record_updated_successfully'), "success");
					return $this->redirect("products");
				}
				else{
					if($db->getLastError()){
						$this->set_page_error();
						$this->write_to_log("edit", "false");
					}
					elseif(!$numRows){
						//not an error, but no record was updated
						$page_error = get_lang('no_record_updated');
						$this->set_page_error($page_error);
						$this->set_flash_msg($page_error, "warning");
						$this->write_to_log("edit", "false");
						return	$this->redirect("products");
					}
				}
			}
		}
		$db->where("products.created_by_admin_id", get_active_user('user_id') );$db->where("products.product_id", $rec_id);;
		$data = $db->getOne($tablename, $fields);
		$page_title = $this->view->page_title = get_lang('edit_products');
		if(!$data){
			$this->set_page_error();
		}
		return $this->render_view("products/edit.php", $data);
	}
	/**
     * Update record field as deleted
	 * Support multi delete by separating record id by comma.
     * @return BaseView
     */
	function delete($rec_id = null){
		Csrf::cross_check();
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$this->rec_id = $rec_id;
		//form multiple delete, split record id separated by comma into array
		$arr_rec_id = array_map('trim', explode(",", $rec_id));
		$db->where("products.product_id", $arr_rec_id, "in");
		$db->where("products.created_by_admin_id", get_active_user('user_id') );
		$modeldata = array(
			"product_status" => "deleted",
			"deleted_at" => datetime_now()
		);
		$bool = $db->update($tablename, $modeldata);
		if($bool){
			$this->write_to_log("delete", "true");
			$this->set_flash_msg(get_lang('record_deleted_successfully'), "success");
		}
		else{
			$page_error = $db->getLastError();
			$this->write_to_log("delete", "false");
			$this->set_flash_msg($page_error, "danger");
		}
		return	$this->redirect("products");
	}
}
