<?php 
/**
 * Users Page Controller
 * @category  Controller
 */
class UsersController extends SecureController{
	function __construct(){
		parent::__construct();
		$this->tablename = "users";
		$this->soft_delete = true;
		$this->delete_field_name = "user_status";
		$this->delete_field_value = "deleted";
	}
	/**
     * List page records
     * @param $fieldname (filter record by a field) 
     * @param $fieldvalue (filter field value)
     * @return BaseView
     */
	function index($fieldname = null , $fieldvalue = null){
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$fields = array("user_id", 
			"user_email", 
			"email_verified_at", 
			"user_mobile_number", 
			"user_provider", 
			"userable_id", 
			"userable_type", 
			"remember_token", 
			"created_at", 
			"updated_at", 
			"user_role_id");
		$pagination = $this->get_pagination(MAX_RECORD_COUNT); // get current pagination e.g array(page_number, page_limit)
		//search table record
		if(!empty($request->search)){
			$text = trim($request->search); 
			$search_condition = "(
				users.user_id LIKE ? OR 
				users.user_email LIKE ? OR 
				users.email_verified_at LIKE ? OR 
				users.user_mobile_number LIKE ? OR 
				users.user_provider LIKE ? OR 
				users.user_password LIKE ? OR 
				users.user_status LIKE ? OR 
				users.userable_id LIKE ? OR 
				users.userable_type LIKE ? OR 
				users.remember_token LIKE ? OR 
				users.created_at LIKE ? OR 
				users.updated_at LIKE ? OR 
				users.user_role_id LIKE ? OR 
				users.deleted_at LIKE ?
			)";
			$search_params = array(
				"%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%","%$text%"
			);
			//setting search conditions
			$db->where($search_condition, $search_params);
			 //template to use when ajax search
			$this->view->search_template = "users/search.php";
		}
		if(!empty($request->orderby)){
			$orderby = $request->orderby;
			$ordertype = (!empty($request->ordertype) ? $request->ordertype : ORDER_TYPE);
			$db->orderBy($orderby, $ordertype);
		}
		else{
			$db->orderBy("users.user_id", ORDER_TYPE);
		}
		$db->where("users.created_by_admin_id", get_active_user('user_id') );
		if($fieldname){
			$db->where($fieldname , $fieldvalue); //filter by a single field name
		}
		$tc = $db->withTotalCount();
		$records = $db->get($tablename, $pagination, $fields);
		$records_count = count($records);
		$total_records = intval($tc->totalCount);
		$page_limit = $pagination[1];
		$total_pages = ceil($total_records / $page_limit);
		$data = new stdClass;
		$data->records = $records;
		$data->record_count = $records_count;
		$data->total_records = $total_records;
		$data->total_page = $total_pages;
		//list of record id for audit trail
		$this->rec_id = array_column($records, "user_id");
		$this->write_to_log("list", "true");
		if($db->getLastError()){
			$this->set_page_error();
			$this->write_to_log("list", "false");
		}
		$page_title = $this->view->page_title = get_lang('users');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		$this->render_view("users/list.php", $data); //render the full page
	}
	/**
     * View record detail 
	 * @param $rec_id (select record by table primary key) 
     * @param $value value (select record by value of field name(rec_id))
     * @return BaseView
     */
	function view($rec_id = null, $value = null){
		$request = $this->request;
		$db = $this->GetModel();
		$rec_id = $this->rec_id = urldecode($rec_id);
		$tablename = $this->tablename;
		$fields = array("user_id", 
			"user_email", 
			"email_verified_at", 
			"user_mobile_number", 
			"user_provider", 
			"userable_id", 
			"userable_type", 
			"remember_token", 
			"created_at", 
			"updated_at", 
			"user_role_id");
		$db->where("users.created_by_admin_id", get_active_user('user_id') );
		if($value){
			$db->where($rec_id, urldecode($value)); //select record based on field name
		}
		else{
			$db->where("users.user_id", $rec_id);; //select record based on primary key
		}
		$record = $db->getOne($tablename, $fields );
		if($record){
			$this->write_to_log("view", "true");
			$page_title = $this->view->page_title = get_lang('view_users');
		$this->view->report_filename = date('Y-m-d') . '-' . $page_title;
		$this->view->report_title = $page_title;
		$this->view->report_layout = "report_layout.php";
		$this->view->report_paper_size = "A4";
		$this->view->report_orientation = "portrait";
		}
		else{
			if($db->getLastError()){
				$this->set_page_error();
			}
			else{
				$this->set_page_error(get_lang('no_record_found'));
			}
			$this->write_to_log("view", "false");
		}
		return $this->render_view("users/view.php", $record);
	}
	/**
     * Insert new record to the database table
	 * @param $formdata array() from $_POST
     * @return BaseView
     */
	function add($formdata = null){
		if($formdata){
			$db = $this->GetModel();
			$tablename = $this->tablename;
			$request = $this->request;
			//fillable fields
			$fields = $this->fields = array("user_id","user_email","user_mobile_number","user_provider","user_password","userable_id","userable_type","user_role_id");
			$postdata = $this->format_request_data($formdata);
			$cpassword = $postdata['confirm_password'];
			$password = $postdata['user_password'];
			if($cpassword != $password){
				$this->view->page_error[] = get_lang('your_password_confirmation_is_not_consistent');
			}
			$this->rules_array = array(
				'user_id' => 'required',
				'user_email' => 'required|valid_email',
				'user_mobile_number' => 'required',
				'user_password' => 'required',
				'userable_id' => 'required',
				'userable_type' => 'required',
				'user_role_id' => 'required',
			);
			$this->sanitize_array = array(
				'user_id' => 'sanitize_string',
				'user_email' => 'sanitize_string',
				'user_mobile_number' => 'sanitize_string',
				'userable_id' => 'sanitize_string',
				'userable_type' => 'sanitize_string',
				'user_role_id' => 'sanitize_string',
			);
			$this->filter_vals = true; //set whether to remove empty fields
			$modeldata = $this->modeldata = $this->validate_form($postdata);
			$password_text = $modeldata['user_password'];
			//update modeldata with the password hash
			$modeldata['user_password'] = $this->modeldata['user_password'] = password_hash($password_text , PASSWORD_DEFAULT);
			$modeldata['user_provider'] = "email";
			$modeldata['created_at'] = datetime_now();
			//Check if Duplicate Record Already Exit In The Database
			$db->where("user_email", $modeldata['user_email']);
			if($db->has($tablename)){
				$this->view->page_error[] = $modeldata['user_email'].get_lang('_already_exist_');
			} 
			if($this->validated()){
				$rec_id = $this->rec_id = $db->insert($tablename, $modeldata);
				if($rec_id){
					$this->write_to_log("add", "true");
					$this->set_flash_msg(get_lang('record_added_successfully'), "success");
					return	$this->redirect("users");
				}
				else{
					$this->set_page_error();
					$this->write_to_log("add", "false");
				}
			}
		}
		$page_title = $this->view->page_title = get_lang('add_new_users');
		$this->render_view("users/add.php");
	}
	/**
     * Update table record with formdata
	 * @param $rec_id (select record by table primary key)
	 * @param $formdata array() from $_POST
     * @return array
     */
	function edit($rec_id = null, $formdata = null){
		$request = $this->request;
		$db = $this->GetModel();
		$this->rec_id = $rec_id;
		$tablename = $this->tablename;
		 //editable fields
		$fields = $this->fields = array("user_id","user_mobile_number","userable_id","userable_type","user_role_id");
		if($formdata){
			$postdata = $this->format_request_data($formdata);
			$this->rules_array = array(
				'user_mobile_number' => 'required',
				'userable_id' => 'required',
				'userable_type' => 'required',
				'user_role_id' => 'required',
			);
			$this->sanitize_array = array(
				'user_mobile_number' => 'sanitize_string',
				'userable_id' => 'sanitize_string',
				'userable_type' => 'sanitize_string',
				'user_role_id' => 'sanitize_string',
			);
			$modeldata = $this->modeldata = $this->validate_form($postdata);
			$modeldata['updated_at'] = datetime_now();
			if($this->validated()){
		$db->where("users.created_by_admin_id", get_active_user('user_id') );
				$db->where("users.user_id", $rec_id);;
				$bool = $db->update($tablename, $modeldata);
				$numRows = $db->getRowCount(); //number of affected rows. 0 = no record field updated
				if($bool && $numRows){
					$this->write_to_log("edit", "true");
					$this->set_flash_msg(get_lang('record_updated_successfully'), "success");
					return $this->redirect("users");
				}
				else{
					if($db->getLastError()){
						$this->set_page_error();
						$this->write_to_log("edit", "false");
					}
					elseif(!$numRows){
						//not an error, but no record was updated
						$page_error = get_lang('no_record_updated');
						$this->set_page_error($page_error);
						$this->set_flash_msg($page_error, "warning");
						$this->write_to_log("edit", "false");
						return	$this->redirect("users");
					}
				}
			}
		}
		$db->where("users.created_by_admin_id", get_active_user('user_id') );$db->where("users.user_id", $rec_id);;
		$data = $db->getOne($tablename, $fields);
		$page_title = $this->view->page_title = get_lang('edit_users');
		if(!$data){
			$this->set_page_error();
		}
		return $this->render_view("users/edit.php", $data);
	}
	/**
     * Update record field as deleted
	 * Support multi delete by separating record id by comma.
     * @return BaseView
     */
	function delete($rec_id = null){
		Csrf::cross_check();
		$request = $this->request;
		$db = $this->GetModel();
		$tablename = $this->tablename;
		$this->rec_id = $rec_id;
		//form multiple delete, split record id separated by comma into array
		$arr_rec_id = array_map('trim', explode(",", $rec_id));
		$db->where("users.user_id", $arr_rec_id, "in");
		$db->where("users.created_by_admin_id", get_active_user('user_id') );
		$modeldata = array(
			"user_status" => "deleted",
			"deleted_at" => datetime_now()
		);
		$bool = $db->update($tablename, $modeldata);
		if($bool){
			$this->write_to_log("delete", "true");
			$this->set_flash_msg(get_lang('record_deleted_successfully'), "success");
		}
		else{
			$page_error = $db->getLastError();
			$this->write_to_log("delete", "false");
			$this->set_flash_msg($page_error, "danger");
		}
		return	$this->redirect("users");
	}
}
